function [fmink, info, noits] = optgain(infile, outfile, outon, diagdata, maxit, method, tolfac)
%
% OPTGAIN	driver program for Robust Fixed-Structure Toolbox
%
%	[fmin, info, noit] = optgain(infile, outfile, outon, diagflag, 
%      maxit, method, tolfac) uses a quasi-newton nonlinear optimization
%	algorithm to solve the optimal control problem given by the 
%	data in the .mat file INFILE.  The resulting optimized gains are
%	saved in the file OUTFILE.mat  
%  
% 	INPUT ARGUMENT DEFINITIONS:

%	 INFILE ....... string. input file containing problem data
%	 OUTFILE ...... string. filename to save optimal gains to
%	 OUTON ........ integer. >= 1.  Print output to screen every
%                     		outon iterations (DEFAULT = 1)
%	 DIAGFLAG ..... 1/0. 1 := diagnostic recorder on, 
%			0 := diagnostic recorder off (DEFAULT = 0)
% 	 MAXIT ........ integer.  Maximum number of iterations to 
%			allow. (DEFAULT = 500)
%	 METHOD ....... 1/2/3. optimization search method. 
%			1 := line search, 2:= double dog-leg, 
%                       3 := hook step. (DEFAULT = 1)
%	 TOLFAC ........ real. >=1. multiplier for code supplied 
%	        	tolerance. (DEFAULT = 1.0)

% 	OUTPUT ARGUMENT DEFINITIONS:
%
%   	 FMIN .......... optimal cost attained
%   	 INFO .......... optimization routine termination code.
%                      	0 := optimal solution found
%                       1 := terminated due to small gradient
%                       2 := terminated due to small stepsize
%                       3 := could not find lower point
%                       4 := MAXIT iteration limit exceeded
%                       5 := too many large steps (unbounded)
%			-1:= insufficient workspace
%      	 NOIT ......... total number of iterations performed 
%                                

% create problem dataset file

     [k0,ctype] = setup(infile);   
%
% set default values for optional arguments
     if ~exist('tolfac');
         tolfac = 1;
     end
     if ~exist('method');
         method = 1;
     end
     if ~exist('maxit');
         maxit = 500;
     end
     if ~exist('diagdata');
         diagdata = 0;
     end
     if ~exist('outon');
         outon = 1; 
     end
     if ~exist('tau');
         tau = 1;
     end
%
% Save input parameters to input file
%
in = fopen('in.dat','w+');
out = fopen('out.dat', 'w+');

%----------------------------------------------------------
% Write dmn to input file
%----------------------------------------------------------
[r,c]=size(k0);
dmn = max(r,c);
fprintf(in, '%i\n', dmn);

%----------------------------------------------------------
% Write k0(i) to input file
%----------------------------------------------------------
for i=1:dmn
  fprintf(in, '%24.16e\n', k0(i));
end

%----------------------------------------------------------
% Print optional arguments to input file
%----------------------------------------------------------
fprintf(in, '%i\n', outon);
fprintf(in, '%i\n', diagdata);
fprintf(in, '%i\n', maxit);
fprintf(in, '%i\n', method);
fprintf(in, '%24.16e\n', tolfac);
fprintf(in, '%i\n', tau);

fclose(in);
fclose(out);

% Call Fortran routine which will read the input parameters
% and make the call to uncmnd.
%
!optfor

% Get output values back
%
load out.dat;

for i=1:dmn
  kmin(i) = out(i,1);
end

j=dmn+1;
fmin = out(j,1);
j=j+1;
info = out(j,1);
j=j+1;
noits = out(j,1);

%
% output the results to the file 'outfile.mat'
%

     load sourcefile

     kstart=1;
     finsavestr=['save ',outfile];

     for i = 1:N

         ki= kmin(kstart:kstart+indexkr(i)*indexkc(i)-1);
         kopti = ivec(ki,indexkr(i),indexkc(i));
         eval(['kopt',num2str(i),'= kopti;']);
         kstart = kstart+indexkr(i)*indexkc(i);
         finsavestr = [finsavestr,' kopt',num2str(i)];

     end;

     if abs(ctype) == 2

        load sourcefile

        zvec = kmin(kstart:kstart+scum(nblk)-1);

        ZZ = [];

        for i = 1:nblk,

                if blk(i,1) == 1,

                        ZZ = daug(ZZ,zvec(scum(i)))

                elseif blk(i,1) == 2,

              ZZ = daug(ZZ,ivec(zvec((scum(i)-3):scum(i)),2,2))

                end

        end

        finsavestr = [finsavestr,' ZZ'];

    elseif abs(ctype) == 3

        load sourcefile

        wvec = kmin(kstart:kstart+scum(nblk)-1);
        zvec = kmin(kstart+scum(nblk):kstart+2*scum(nblk)-1);

        WW = [];
        ZZ = [];

        for i = 1:nblk,
          
            if blk(i,1) == 1,

               WW = daug(WW,wvec(scum(i)));
               ZZ = daug(ZZ,zvec(scum(i)));

            elseif blk(i,1) == 2,

            WW = daug(WW,ivec(wvec((scum(i)-3):scum(i)),2,2));
            ZZ = daug(ZZ,ivec(zvec((scum(i)-3):scum(i)),2,2));

            end

        end

        finsavestr = [finsavestr,' WW ZZ'];

    end

    finsavestr = [finsavestr,' fmin info'];

    eval(finsavestr);

end

