      subroutine rootnf(n,nfe,iflag,relerr,abserr,y,yp,yold,ypold,
     &   a,qr,alpha,tz,pivot,w,wp,par,ipar)
c
c rootnf  finds the point  ybar = (1, xbar)  on the zero curve of the
c homotopy map.  it starts with two points yold=(lambdaold,xold) and
c y=(lambda,x) such that  lambdaold < 1 <= lambda , and alternates
c between hermite cubic interpolation and newton iteration until
c convergence.
c
c on input:
c
c n = dimension of x and the homotopy map.
c
c nfe = number of jacobian matrix evaluations.
c
c iflag = -2, -1, or 0, indicating the problem type.
c
c relerr, abserr = relative and absolute error values.  the iteration is
c    considered to have converged when a point y=(lambda,x) is found 
c    such that
c
c    |y(1) - 1| <= relerr + abserr              and
c
c    ||z|| <= relerr*||x|| + abserr  ,          where
c
c    (?,z) is the newton step to y=(lambda,x).
c
c y(1:n+1) = point (lambda(s), x(s)) on zero curve of homotopy map.
c
c yp(1:n+1) = unit tangent vector to the zero curve of the homotopy map
c    at  y .
c
c yold(1:n+1) = a point different from  y  on the zero curve.
c
c ypold(1:n+1) = unit tangent vector to the zero curve of the homotopy
c    map at  yold .
c
c a(1:*) = parameter vector in the homotopy map.
c
c qr(1:n,1:n+2), alpha(1:n), tz(1:n+1), pivot(1:n+1), w(1:n+1), 
c    wp(1:n+1)  are work arrays used for the qr factorization (in the
c    newton step calculation) and the interpolation.
c
c par(1:*) and ipar(1:*) are arrays for (optional) user parameters,
c    which are simply passed through to the user written subroutines
c    rho, rhojac.
c
c on output:
c
c n , relerr , abserr , a  are unchanged.
c
c nfe  has been updated.
c
c iflag 
c    = -2, -1, or 0 (unchanged) on a normal return.
c
c    = 4 if a jacobian matrix with rank < n has occurred.  the
c        iteration was not completed.
c
c    = 6 if the iteration failed to converge.  y  and  yold  contain
c        the last two points found on the zero curve.
c
c y  is the point on the zero curve of the homotopy map at  lambda = 1 .
c
c
c calls  d1mach , dnrm2 , root , tangnf .
c
      double precision abserr,aerr,d1mach,dd001,dd0011,dd01,dd011,
     &   dels,dnrm2,f0,f1,fp0,fp1,qofs,qsout,relerr,rerr,s,sa,sb,
     &   sout,u
      integer iflag,judy,jw,lcode,limit,n,nfe,np1
c
c ***** array declarations. *****
c
      double precision y(n+1),yp(n+1),yold(n+1),ypold(n+1),a(n),
     &   qr(n,n+2),alpha(n),tz(n+1),w(n+1),wp(n+1),par(1)
      integer pivot(n+1),ipar(1)
c
c ***** end of dimensional information. *****
c
c the limit on the number of iterations allowed may be changed by
c changing the following parameter statement:
      parameter (limit=20)
c
c definition of hermite cubic interpolant via divided differences.
c
      dd01(f0,f1,dels)=(f1-f0)/dels
      dd001(f0,fp0,f1,dels)=(dd01(f0,f1,dels)-fp0)/dels
      dd011(f0,f1,fp1,dels)=(fp1-dd01(f0,f1,dels))/dels
      dd0011(f0,fp0,f1,fp1,dels)=(dd011(f0,f1,fp1,dels) - 
     &                            dd001(f0,fp0,f1,dels))/dels
      qofs(f0,fp0,f1,fp1,dels,s)=((dd0011(f0,fp0,f1,fp1,dels)*(s-dels) +
     &   dd001(f0,fp0,f1,dels))*s + fp0)*s + f0
c
c
      u=d1mach(4)
      rerr=max(relerr,u)
      aerr=max(abserr,0.0d0)
      np1=n+1
c
c *****  main loop.  *****
c
100   do 300 judy=1,limit
      do 110 jw=1,np1
        tz(jw)=y(jw)-yold(jw)
110   continue
      dels=dnrm2(np1,tz,1)
c     write(6,303) y(1),yold(1),dels
c303   format(/,d25.15,2x,d25.15,2x,d25.15)
c
c using two points and tangents on the homotopy zero curve, construct 
c the hermite cubic interpolant q(s).  then use  root  to find the s
c corresponding to  lambda = 1 .  the two points on the zero curve are
c always chosen to bracket lambda=1, with the bracketing interval
c always being [0, dels].
c
      sa=0.0
      sb=dels
      lcode=1
130   call root(sout,qsout,sa,sb,rerr,aerr,lcode)
      if (lcode .gt. 0) go to 140
      qsout=qofs(yold(1),ypold(1),y(1),yp(1),dels,sout) - 1.0
      go to 130
c if lambda = 1 were bracketed,  root  cannot fail.
140   if (lcode .gt. 2) then
        print*,'lcode',lcode
        iflag=6
        return
      endif
c
c calculate q(sa) as the initial point for a newton iteration.
      do 150 jw=1,np1
        w(jw)=qofs(yold(jw),ypold(jw),y(jw),yp(jw),dels,sa)
150   continue
c calculate newton step at q(sa).
      call tangnf(sa,w,wp,ypold,a,qr,alpha,tz,pivot,nfe,n,iflag,
     &            par,ipar)
      if (iflag .gt. 0) return
c next point = current point + newton step.
      do 160 jw=1,np1
        w(jw)=w(jw)+tz(jw)
160   continue
c get the tangent  wp  at  w  and the next newton step in  tz .
      call tangnf(sa,w,wp,ypold,a,qr,alpha,tz,pivot,nfe,n,iflag,
     &            par,ipar)
      if (iflag .gt. 0) return
c take newton step and check convergence.
      do 170 jw=1,np1
        w(jw)=w(jw)+tz(jw)
170   continue
      if ((abs(w(1)-1.0) .le. rerr+aerr) .and.
     &    (dnrm2(n,tz(2),1) .le. rerr*dnrm2(n,w(2),1)+aerr)) then
        do 180 jw=1,np1
          y(jw)=w(jw)
180     continue
        return
      endif
c if the iteration has not converged, discard one of the old points
c such that  lambda = 1  is still bracketed.
      if ((yold(1)-1.0)*(w(1)-1.0) .gt. 0.0) then
        do 200 jw=1,np1
          yold(jw)=w(jw)
          ypold(jw)=wp(jw)
200     continue
      else
        do 210 jw=1,np1
          y(jw)=w(jw)
          yp(jw)=wp(jw)
210     continue
      endif
300   continue
c
c ***** end of main loop. *****
c
c the alternating osculatory cubic interpolation and newton iteration
c has not converged in  limit  steps.  error return.
      iflag=6
      return
      end
