function dsofformat(filename,ncvec,noivec,A,Bu,Bw,Cy,Cz,Dyu,Dyw,Dzu,Dzw,Ma,Na,Mb,Nb,Mc,Nc)

% DSOFFORMAT	formulation of system matrices to decentralized 
%		static output feedback format for use with OPTGAIN.
%
%		dsofformat(filename,ncvec,noivec, A,Bu,Bw,Cy,Cz,
%		Dyu,Dyw,Dzu,Dzw) reformulates the data contained in
%		the arguments into a K-channel decentralized static 
%		output feedback problem, and saves the resulting 
%		matrices to the file FILENAME.mat
%	
%		NCVEC is a K x 1 whose i_th element is the
%		order of the dynamics for the K_th processor.  Static 
%		compensators are indicated by a dimension of 0.  
%		NOIVEC is a K x 2 matrix. The 
%		first column contains the dimension of measurement  
%		signals for each channel, while the second column 
%		contains the dimension of the actuator signals for the
%		K_th channel. 
%
%		dsofformat(filename,ncvec,noivec, A,Bu,Bw,Cy,Cz,
%		Dyu,Dyw,Dzu,Dzw,Ma,Na,Mb,Nb,Mc,Nc) will create
%		the matrices for handling uncertainty in the plant of
%		the form 
%
%		  | A + Ma * Delta_A * Na | B + Mb * Delta_B * Nb  |
%		  |-----------------------+----------------------  |
%		  | C + Mc * Delta_C * Nc |         Dyu            |
%
%		if any of the uncertainties are not present, the 
%		corresponding M_ and N_ should be entered as NULL ([]).
%

%
% check for appropriate dimensions

	if size(ncvec,2) ~= 1,

		error('NCVEC must be a K x 1 vector')

	end

	if size(ncvec,1) ~= size(noivec,1)

		error('NCVEC and NOIVEC must have the same number of rows')

	end

	if sum(noivec(:,1)) ~= size(Cy,1)

		error('Number & Dimension of output channels does not agree with dimension of Cz')

	end

	if sum(noivec(:,2)) ~= size(Bu,2)

		error('Number & Dimension of input channels does not agree with dimension of Bu')

	end

%
% begin formatting

     K  = length(ncvec);
     np = size(A,1);
     [nz,nw] = size(Dzw);

     ncsum = sum(ncvec);
     ncl = np + ncsum;

     a0  = zeros(ncl, ncl);
     a0(1:np,1:np) = A;

     bw = zeros(ncl, nw);
     bw(1:np,1:nw) = Bw;
 
     cz = zeros(nz, ncl);
     cz(1:nz,1:np) = Cz;

     if nargin > 12

		nda = size(Ma,2);
		ndb = size(Mb,2);
		ndc = size(Mc,2);

		nsumd = nda + ndb + ndc;

		bd = zeros(ncl,nsumd);

		if nda > 0

			bd(1:np,1:nda) = Ma;

		end

		if ndb > 0 

			bd(1:np,nda+1:nda+ndb) = Mb;

		end

		ce = zeros(nsumd,ncl);

		if nda > 0 

			ce(1:nda,1:np) = Na;

		end
		
		if ndc > 0 

			ce(nda + ndb + 1:nsumd,1:np) = Nc;

		end

    end

     kindex  = [];
     indexkr = [];
     indexkc = [];

     knum = 1;
     nistart = 1;
     nostart = 1;
     ncstart = np + 1;
     N = 0;

     for i = 1:K
	
	nci = ncvec(i);
	nii = noivec(i,1);
	noi = noivec(i,2);

	Bui  = Bu(1:np, nistart:nistart + nii - 1);
	Cyi  = Cy(nostart:nostart + noi - 1, 1:np);
	Dyiw = Dyw(nostart:nostart + noi - 1, 1:nw);
	Dzui = Dzu(1:nz, nistart:nistart + nii -1);


% Case 1: Dynamic Compensators:

	if nci ~= 0

% zero out matrices which correspond to Aci, then
% replace non-zero entries appropriately
%
		buknum = zeros(ncl, nci);
		cyknum = zeros(nci, ncl);
		dywknum = zeros(nci, nw);
		dzuknum = zeros(nz, nci);

		buknum(ncstart:ncstart + nci - 1, 1:nci) = eye(nci);
		cyknum(1:nci, ncstart:ncstart + nci - 1) = eye(nci);

		eval(['bu',num2str(knum),'  = buknum;']);
		eval(['cy',num2str(knum),'  = cyknum;']);
		eval(['dy',num2str(knum),'w = dywknum;']);
		eval(['dzu',num2str(knum),' = dzuknum;']);

% zero out matrices which correspond to Bci, then
% replace non-zero entries appropriately
%
		buknum_1 = zeros(ncl, nci);
		cyknum_1 = zeros(noi, ncl);
		dywknum_1 = zeros(noi, nw);
		dzuknum_1 = zeros(nz, nci);

		buknum_1(ncstart:ncstart + nci -1,1:nci) = eye(nci);
		cyknum_1(1:noi, 1:np) = Cyi; 
		dywknum_1 = Dyiw;

		eval(['bu',num2str(knum + 1),'= buknum_1;']);
		eval(['cy',num2str(knum + 1),'= cyknum_1;']);
		eval(['dy',num2str(knum + 1),'w = dywknum_1;']);
		eval(['dzu',num2str(knum + 1),' = dzuknum_1;']);

% zero out matrices which correspond to Cci, then
% replace non-zero entries appropriately
%
		buknum_2 = zeros(ncl, nii);
		cyknum_2 = zeros(nci, ncl);
		dywknum_2 = zeros(nci, nw);
		dzuknum_2 = zeros(nz, nii);

		buknum_2(1:np, 1:nii) = Bui;
		cyknum_2(1:nci, ncstart:ncstart + nci - 1) = eye(nci);
		dzuknum_2 = Dzui;

		eval(['bu',num2str(knum + 2),'= buknum_2;']);
		eval(['cy',num2str(knum + 2),'= cyknum_2;']);
		eval(['dy',num2str(knum + 2),'w = dywknum_2;']);
		eval(['dzu',num2str(knum + 2),' = dzuknum_2;']);

% create uncertainty matrices, if necessary
%

		if nargin > 12

			dydknum1 = zeros(nci,nsumd);
			dydknum2 = zeros(noi,nsumd);
			dydknum3 = zeros(nci,nsumd);

			deuknum1 = zeros(nsumd,nci);
			deuknum2 = zeros(nsumd,nci);
			deuknum3 = zeros(nsumd,nii);

			if ndc > 0

				Mci = Mc(nostart:nostart + noi - 1, 1:ndc);
				dydknum2(:,nda + ndb + 1:nsumd) = Mci;

			end
			
			if ndb > 0


				Nbi = Nb(1:ndb, nistart:nistart + nii - 1);
				deuknum3(nda + 1:nda + ndb,:) = Nbi;

			end

			eval(['dy',num2str(knum),'d = dydknum1;']);
			eval(['dy',num2str(knum + 1),'d = dydknum2;']);
			eval(['dy',num2str(knum + 2),'d = dydknum3;']);

			eval(['deu',num2str(knum),' = deuknum1;']);
			eval(['deu',num2str(knum + 1),' = deuknum2;']);
			eval(['deu',num2str(knum + 2),' = deuknum3;']);

		end
			
% update index vectors
%
		kindex = [kindex,1,1,1];
		indexkr = [indexkr,nci,nci,nii];
		indexkc = [indexkc,nci,noi,nci];

% increase counters and repeat
%
		nistart = nistart + nii;
		nostart = nostart + noi;
		ncstart = ncstart + nci;
		knum = knum + 3;

% Case 2: Static Compensators

	elseif nci == 0

		buknum = Bui;
		cyknum = Cyi;
		dywknum = Dyiw;
		dzuknum = Dzui;

		eval(['bu',num2str(knum),'= buknum;']);
		eval(['cy',num2str(knum),'= cyknum;']);
		eval(['dy',num2str(knum),'w = dywknum;']);
		eval(['dzu',num2str(knum),' = dzuknum;']);

% update index vectors
%
		kindex = [kindex,1];
		indexkr = [indexkr,nii];
		indexkc = [indexkc,noi];

% increase counters and repeat
%
		nistart = nistart + nii;
		nostart = nostart + noi;
		ncstart = ncstart + nci;
		knum = knum + 1;

	end

     end

     N = knum-1;

     savestr = ['save ',filename];

     if max(ncvec) ~= 0

     	clear buknum_1 cyknum_1 dywknum_1 dzuknum_1
     	clear buknum_2 cyknum_2 dywknum_2 dzuknum_2

     end

     Dzd = zeros(nz,nsumd);

     if nargin > 12

     	Dew = zeros(nsumd,nw);
     	Ded = zeros(nsumd,nsumd);

     end

     clear i filename
     clear K ncvec noivec A Bu Cy Bw Dyw Cz Dzu Dyu
     clear nistart nostart ncstart knum ncsum ncl
     clear nii noi nci Bui Cyi Dzui Dyiw 
     clear buknum cyknum dywknum dzuknum

     clear dydknum1 dydknum2 dydknum3 deuknum1 deuknum2 deuknum3
     clear nda ndb ndc nsumd
     clear Ma Mb Mc Na Nb Nc

     nrk = kindex*indexkr';
     nck = kindex*indexkc';
     
     Dyu = zeros(nck,nrk);
     A = a0;

     nrstart = 1;
     ncstart = 1;
     q = 1;

     Bu = [];
     Cy = [];
     Dyw = [];
     Dzu = [];
     Bw = bw;
     Cz = cz;

     if nargin > 12

     	Deu = [];
     	Dyd = [];
     	Bd = bd;
     	Ce = ce;

     end

     for i = 1:N

	for j = 1:kindex(i)

		indi = num2str(q);
		ijstr = [num2str(i),num2str(j)];

		bustr = ['bui = bu',indi,';'];
		cystr = ['cyi = cy',indi,';'];
		dywstr = ['dyiw = dy',indi,'w;'];
		dzustr = ['dzui = dzu',indi,';'];

		eval(bustr)
		eval(cystr)
		eval(dywstr)
		eval(dzustr)

		nri = indexkr(i);
		nci = indexkc(i);

		Bu = [Bu,bui];
		Cy = [Cy;cyi];
		Dyw = [Dyw;dyiw];
		Dzu = [Dzu,dzui];

		if nargin > 12

			deustr = ['deui = deu',indi,';'];
			dydstr = ['dyid = dy',indi,'d;'];

			eval(dydstr)
			eval(deustr)

			Dyd = [Dyd;dyid];
			Deu = [Deu,deui];

		end

		Ql = zeros(nrk,nri);
		Qr = zeros(nci,nck);

		Ql(nrstart:nrstart+nri-1,:) = eye(nri);
		Qr(:,ncstart:ncstart+nci-1) = eye(nci);

		qlstr = ['QL',ijstr,' = Ql;'];
		qrstr = ['QR',ijstr,' = Qr;'];

		savestr = [savestr,' QL',ijstr,' QR',ijstr];

		eval(qlstr);
		eval(qrstr);

		q = q+1;
		ncstart = ncstart + nci;
		nrstart = nrstart + nri;

	  end

     end

     savestr =  [savestr,' A Bu Bw Cy Dyu Dyw Cz Dzu Dzw kindex indexkr indexkc N'];

     if nargin > 12

		savestr = [savestr,' Bd Ce Dyd Ded Deu Dew'];

     end
 
     eval(savestr);


	
	
	
	
